{% extends "core_main_app/_render/admin/theme/tools/section.html" %}
{% load staticfiles %}

{% block section_content %}
    {% include 'core_main_app/admin/commons/errors/errors.html' %}
{% endblock %}